indexing

	description: "Class that implements whatever an HTTP 1.0 client can do."

	author: "Berend de Boer"
	date: "$Date: 2003/01/09 $"
	revision: "$Revision: #1 $"


class

	EPX_HTTP_10_CLIENT


inherit

	EPX_HTTP_CLIENT

	EPX_MIME_FIELD_NAMES
		export
			{NONE} all
		end


creation

	make,
	make_from_port,
	make_from_host,
	make_from_host_and_port


feature -- Client http version

	client_version: STRING is
		once
			Result := "HTTP/1.0"
		end


feature -- Requests

	get (path: STRING) is
			-- Send GET request to server.
		local
			request: STRING
		do
			assert_http_closed
			request := sh.make_with_capacity (128)
			request.append_string (once_get)
			request.append_string (path)
			request.append_character (' ')
			request.append_string (client_version)
			request.append_character ('%R')
			request.append_character ('%N')
			request.append_string ("Host: ")
			request.append_string (host.name)
			request.append_character ('%R')
			request.append_character ('%N')
			append_other_fields (request)
			request.append_string (once_connection_close)
			request.append_string (null_line)
			create http.open_by_address (sa)
			http.write_string (request)
		end

	head (path: STRING) is
			-- Send HEAD request to server.
			-- `path' should not include http: and the host name, only
			-- the page that is requested. Any query and fragment parts are ok.
		require
			path_not_empty: path /= Void and then not path.is_empty
		local
			request: STRING
		do
			assert_http_closed
			request := sh.make_with_capacity (128)
			request.append_string (once_head)
			request.append_string (path)
			request.append_character (' ')
			request.append_string (client_version)
			request.append_character ('%R')
			request.append_character ('%N')
			request.append_string ("Host: ")
			request.append_string (host.name)
			request.append_character ('%R')
			request.append_character ('%N')
			append_other_fields (request)
			request.append_string (once_connection_close)
			request.append_string (null_line)
			create http.open_by_address (sa)
			http.write_string (request)
		end

	options (path: STRING) is
			-- Get server options. Path is required when the request is
			-- being made to a proxy.
		local
			request: STRING
		do
			assert_http_closed
			request := sh.make_with_capacity (128)
			if path = Void or else path.is_empty then
				request.append_string ("OPTIONS * ")
			else
				request.append_string ("OPTIONS ")
				request.append_string (path)
				request.append_character (' ')
			end
			request.append_string (client_version)
			request.append_character ('%R')
			request.append_character ('%N')
			request.append_string ("Host: ")
			request.append_string (host.name)
			request.append_character ('%R')
			request.append_character ('%N')
			append_other_fields (request)
			request.append_string (once_connection_close)
			request.append_string (null_line)
			create http.open_by_address (sa)
			http.write_string (request)
		end


feature {NONE} -- Request implementation

	append_other_fields (request: STRING) is
		do
			request.append_string ("MIME-Version: 1.0%R%N")
			if user_agent /= Void then
				request.append_string ("User-Agent: ")
				request.append_string (user_agent)
				request.append_character ('%R')
				request.append_character ('%N')
			end
		end


feature -- Fields that are send with a request if set

	user_agent: STRING
			-- Identification of client program.
			-- Common examples are:
			--   Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)
			--   Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.0.0) Gecko/20020529
			--   Microsoft Internet Explorer

	set_user_agent (value: STRING) is
			-- Set the client identification.
		require
			value_void_or_not_empty: value = Void or else not value.is_empty
		do
			user_agent := value
		ensure
			user_agent_set:
				(value = Void implies user_agent = Void) or else
				(user_agent.is_equal (value))
		end


feature -- Response

	body: EPX_MIME_BODY_TEXT is
			-- Return body as text, if applicable, else Void.
		do
			Result ?= parser.part.body
		end

	fields: DS_HASH_TABLE [EPX_MIME_FIELD, STRING] is
			-- Header fields of response.
		do
			Result := parser.part.fields
		end

	is_response_ok: BOOLEAN is
			-- Does the returned `response_code' indicate success?
		do
			Result := response_code >= 200 and response_code <= 299
		end

	part: EPX_MIME_PART is
			-- The entire parsed MIME message
		do
			Result := parser.part
		end

	c: CHARACTER
	read_response is
			-- Read entire resonse, parse while reading.
		do
			-- first line is HTTP version
			if parser = Void then
				create parser.make_from_file_descriptor (http)
			else
				parser.make_from_file_descriptor (http)
			end
			read_and_parse_status_line
			parser.parse
			if parser.syntax_error then
				response_code := 500
				response_phrase := "Syntax error parsing response."
			end
		end

	response_code: INTEGER

	response_phrase: STRING

	server_version: STRING
			-- Set by `read_response'.


feature -- Individual response fields, Void if not available

	location: STRING is
		do
			fields.search (field_name_location)
			if fields.found then
				Result := fields.found_item.value
			end
		end


feature {NONE} -- Implementation

	read_and_parse_status_line is
			-- First line is HTTP version and response.
			-- But not always, there's some shit out there that
			-- immediately starts with the MIME headers. I don't try to
			-- recover from that.
		local
			p, q: INTEGER
			s: STRING
			status_line: STRING
		do
			status_line := sh.make_with_capacity (64)
			from
				parser.read_character
			until
				parser.last_character = '%N' or else
				parser.eof
			loop
				status_line.append_character (parser.last_character)
				parser.read_character
			end

			-- Removing %R if part of CRLF
			if status_line.count > 0 and then status_line.item (status_line.count) = '%R' then
				status_line.remove (status_line.count)
			end

			p := status_line.index_of (' ', 1)
			if p > 1 then
				q := status_line.index_of (' ', p+1)
			end
			if p > 1 and q > 1 then
				server_version := status_line.substring (1, p-1)
				sh.trim (server_version)
				s := status_line.substring (p+1, q-1)
				sh.trim (s)
				if s.is_integer then
					response_code := s.to_integer
				else
					response_code := 500
				end
				response_phrase := status_line.substring (q+1, status_line.count)
				sh.trim (response_phrase)
			else
				-- Can we recover from garbage??
				response_code := 500
				response_phrase := ""
				server_version := ""
			end
		end


feature {NONE} -- MIME parser

	parser: EPX_MIME_PARSER


feature {NONE} -- Once strings

	once_get: STRING is "GET "

	once_head: STRING is "HEAD "

	once_connection_close: STRING is "Connection: close"

	null_line: STRING is "%R%N%R%N"

end
