indexing

	description: "Standard C dynamic memory. Memory should be at least 1 byte%
	%large. 0 size memory is not supported."

	note: "If you treat this as an array, it is zero based."
	author: "Berend de Boer"
	date: "$Date: 2003/01/09 $"
	revision: "$Revision: #4 $"


class

	STDC_BUFFER

inherit

	STDC_HANDLE [POINTER]
		rename
			close as deallocate,
			handle as ptr,
			is_open as is_allocated
		redefine
			do_close
		end

	STDC_SYSTEM
		export
			{NONE} all
		end

	CAPI_STDLIB
		export
			{NONE} all
		end

	CAPI_STRING
		export
			{NONE} all
		end

	EPX_POINTER_ADD
		export
			{NONE} all
		end


creation

	allocate,
	allocate_and_clear,
	make_from_pointer


feature -- Creation

	allocate (a_capacity: INTEGER) is
			-- allocate memory of `a_capacity' bytes
		require
			valid_capacity: a_capacity > 0
			allocation_of_capacity_allowed: security.memory.is_allowed_allocation (a_capacity)
			memory_bounds_not_exceeded: security.memory.is_within_maximum (a_capacity)
		local
			new_ptr: POINTER
		do
			security.memory.check_allocation (a_capacity)
			if is_owner then
				deallocate
			elseif is_allocated then
				detach
			end
			new_ptr := posix_malloc (a_capacity)
			if new_ptr = default_pointer then
				raise_posix_error
			else
				capacity := a_capacity
				set_handle (new_ptr, True)
			end
		ensure
			allocated: is_allocated
			owner: is_owner
			capacity_set: capacity = a_capacity
			tracked_memory_increased:
				security.memory.allocated_memory = old security.memory.allocated_memory + capacity
		end

	allocate_and_clear (a_capacity: INTEGER) is
			-- allocate memory of `a_capacity' bytes, make sure it's zeroed out
		require
			valid_capacity: a_capacity > 0
			allowed: security.memory.is_allowed_allocation (a_capacity)
			memory_bounds_not_exceeded: security.memory.is_within_maximum (a_capacity)
		local
			new_ptr: POINTER
		do
			security.memory.check_allocation (a_capacity)
			if is_owner then
				deallocate
			elseif is_allocated then
				detach
			end
			new_ptr := posix_calloc (a_capacity, 1)
			if new_ptr = default_pointer then
				raise_posix_error
			else
				capacity := a_capacity
				set_handle (new_ptr, True)
			end
		ensure
			allocated: is_allocated
			owner: is_owner
			capacity_set: capacity = a_capacity
			tracked_memory_increased:
				security.memory.allocated_memory = old security.memory.allocated_memory + capacity
		end

	make_from_pointer (
			a_pointer: POINTER; a_capacity: INTEGER;
			a_become_owner: BOOLEAN) is
			-- attach a pointer to this object. If `a_become_owner' is
			-- True, it will deallocate the pointer when `close' is
			-- called, or when this object is garbage collected.
		require
			valid_capacity: a_capacity > 0
			valid_pointer: a_pointer /= default_pointer
			allowed: a_become_owner implies security.memory.is_allowed_allocation (a_capacity)
		do
			if a_become_owner then
				security.memory.check_allocation (a_capacity)
			end
			if is_owner then
				deallocate
			elseif is_allocated then
				detach
			end
			capacity := a_capacity
			attach_to_handle (a_pointer, a_become_owner)
		ensure
			allocated: is_allocated
			pointer_set: ptr = a_pointer
			capacity_set: capacity = a_capacity
			owner_set: a_become_owner = is_owner
			resource_accounted_for:
				(a_become_owner implies
					resource_count = old resource_count + capacity) and then
				(not a_become_owner implies
					resource_count = old resource_count)
		end


feature -- Other allocation commands

	resize (new_capacity: INTEGER) is
			-- Resize memory to `new_capacity' bytes. Expanded memory is not
			-- guaranteed to be zeroed out.
		require
			valid_capacity: new_capacity > 0
			allocated: is_allocated
			may_resize: is_owner
			allowed: security.memory.is_allowed_allocation (new_capacity)
			memory_bounds_not_exceeded: new_capacity > capacity implies security.memory.is_within_maximum (new_capacity - capacity)
		local
			new_ptr: POINTER
		do
			security.memory.check_allocation (new_capacity)
			new_ptr := posix_realloc (ptr, new_capacity)
			if new_ptr = default_pointer then
				raise_posix_error
			else
				clear_handle
				capacity := new_capacity
				set_handle (new_ptr, True)
			end
		ensure
			successfull_allocation: is_allocated
			capacity_set: security.error_handling.exceptions_enabled implies capacity >= new_capacity
			tracked_memory_changed:
				security.error_handling.exceptions_enabled implies
					(security.memory.allocated_memory = (old security.memory.allocated_memory - old capacity) + new_capacity)
		end

	realloc (new_capacity: INTEGER) is
		obsolete "Use resize instead."
		do
			resize (new_capacity)
		end


feature -- Access

	resource_usage_can_be_increased: BOOLEAN is
			-- Can the number of allocated resources increased with `capacity'?
		do
			Result := security.memory.is_within_maximum (capacity)
		end


feature -- Copy data internally or externally

	copy_from (source: STDC_BUFFER; src_offset, dest_offset, bytes: INTEGER) is
			-- Move data from another buffer into ourselves.
			-- Start at offset `src_offset', into
			-- offset `dest_offset', moving `bytes' bytes
			-- Memory may overlap.
		require
			valid_source: source /= Void
			valid_source_offset: src_offset >= 0 and
										src_offset + bytes <= source.capacity
			valid_dest_offset: dest_offset >= 0 and
									 dest_offset + bytes <= capacity
			valid_bytes_to_move: bytes >= 0
		do
			if source.ptr = Current.ptr then
				move (src_offset, dest_offset, bytes)
			else
				memory_move (source.ptr, src_offset, dest_offset, bytes)
			end
		end

	memory_copy (source: POINTER; src_offset: INTEGER; dest_offset, bytes: INTEGER) is
			-- Copy data from `source', offset `src_offset', to location
			-- `dest_offset' in this buffer, for `bytes' bytes.
			-- Memory may not overlap, use `move' to copy within buffer
			-- or `memory_move' to copy from potentially overlapping buffer.
		require
			my_memory_does_not_overlap: True
			I_know_what_I_am_doing: True
			valid_source: source /= default_pointer
			dest_offset_not_negative: dest_offset >= 0
			data_should_fit: dest_offset + bytes <= capacity
			valid_bytes_to_move: bytes >= 0
		local
			src,
			dest: POINTER
		do
			src := posix_pointer_add (source, src_offset)
			dest := posix_pointer_add (ptr, dest_offset)
			posix_memcpy (dest, src, bytes)
		end

	memcpy (source: POINTER; src_offset: INTEGER; dest_offset, bytes: INTEGER) is
		obsolete "Use memory_copy instead."
		do
			memory_copy (source, src_offset, dest_offset, bytes)
		end

	memory_move (source: POINTER; src_offset: INTEGER; dest_offset, bytes: INTEGER) is
			-- Copy data from `source', offset `src_offset', to location
			-- `dest_offset' in this buffer, for `bytes' bytes.
			-- Memory may overlap.
		require
			valid_source: source /= default_pointer
			data_should_fit: dest_offset + bytes <= capacity
			valid_bytes_to_move: bytes >= 0
		local
			src,
			dest: POINTER
		do
			src := posix_pointer_add (source, src_offset)
			dest := posix_pointer_add (ptr, dest_offset)
			posix_memmove (dest, src, bytes)
		end

	memmove (source: POINTER; src_offset: INTEGER; dest_offset, bytes: INTEGER) is
		obsolete "Use memory_move instead."
		do
			memory_move (source, src_offset, dest_offset, bytes)
		end

	move (src_offset, dest_offset: INTEGER; bytes: INTEGER) is
			-- Move data around in buffer itself from offset `src_offset' to
			-- offset `dest_offset', moving `bytes' bytes.
			-- Memory may overlap.
		require
			valid_source_offset: src_offset >= 0 and src_offset + bytes <= capacity
			valid_dest_offset: dest_offset >= 0 and dest_offset + bytes <= capacity
		local
			src,
			dest: POINTER
		do
			src := posix_pointer_add (ptr, src_offset)
			dest := posix_pointer_add (ptr, dest_offset)
			posix_memmove (dest, src, bytes)
		end


feature -- Set/get bytes (8-bit data)

	peek_byte (index: INTEGER): INTEGER is
		obsolete "use peek_uint8 instead."
		do
			Result := peek_uint8 (index)
		end

	peek_tinyint (index: INTEGER): INTEGER is
		obsolete "use peek_int8 instead."
		do
			Result := peek_int8 (index)
		end

	poke_byte (index, value: INTEGER) is
		obsolete "use poke_uint8 instead."
		do
			poke_uint8 (index, value)
		end

	poke_tinyint (index, value: INTEGER) is
		obsolete "use poke_int8 instead."
		do
			poke_int8 (index, value)
		end

	peek_uint8, infix "@" (index: INTEGER): INTEGER is
			-- consider memory an array of 8 bit values.
		require
			valid_index: is_valid_index (index)
		do
			Result := posix_peek_uint8 (ptr, index)
		ensure
			possible_values: Result >= 0 and Result < 256
		end

	poke_uint8 (index, value: INTEGER) is
		require
			valid_index: is_valid_index (index)
			valid_byte: value >= 0 and value < 256
		do
			posix_poke_uint8 (ptr, index, value)
		ensure
			consistent: peek_uint8 (index) = value
		end

	peek_int8 (index: INTEGER): INTEGER is
			-- consider memory an array of 8 bit values.
		require
			valid_index: is_valid_index (index)
		local
			byte_value: INTEGER
		do
			byte_value := posix_peek_uint8 (ptr, index)
			if byte_value > 127 then
				Result := byte_value - 256
			else
				Result := byte_value
			end
		ensure
			possible_values: Result >= -128 and Result <= 127
		end

	poke_int8 (index, value: INTEGER) is
		require
			valid_index: is_valid_index (index)
			valid_tinyin: value >= -128 and value <= 127
		local
			byte_value: INTEGER
		do
			if value < 0 then
				byte_value := 256 + value
			else
				byte_value := value
			end
			posix_poke_uint8 (ptr, index, byte_value)
		ensure
			consistent: peek_int8 (index) = value
		end


feature -- Set/get integers (16-bit data)

	peek_int16, peek_int16_native (index: INTEGER): INTEGER is
			-- Read signed 16 bit value at offset `index' in native
			-- endian format.
		require
			valid_index: is_valid_range (index, index + 1)
		do
			Result := posix_peek_int16_native (ptr, index)
		ensure
			valid_result: Result >= -32768 and Result <= 32767
		end

	peek_uint16, peek_uint16_native (index: INTEGER): INTEGER is
			-- Read unsigned 16 bit value at offset `index' in native format.
		require
			valid_index: is_valid_range (index, index + 1)
		do
			Result := posix_peek_uint16_native (ptr, index)
		ensure
			valid_result: Result >= 0 and Result <= 65535
		end

	peek_int16_big_endian (index: INTEGER): INTEGER is
			-- Read 16 bit value at offset `index' in big endian format.
		require
			valid_index: is_valid_range (index, index + 1)
		local
			i: INTEGER
		do
			i := posix_peek_int16_native (ptr, index)
			if is_little_endian then
				i := posix_swap16 (i)
			end
			Result := i
		ensure
			valid_result: Result >= -32768 and Result <= 32767
		end

	peek_int16_little_endian (index: INTEGER): INTEGER is
			-- Read 16 bit value at offset `index' in little endian format.
		require
			valid_index: is_valid_range (index, index + 1)
		local
			i: INTEGER
		do
			i := posix_peek_int16_native (ptr, index)
			if is_big_endian then
				i := posix_swap16 (i)
			end
			Result := i
		ensure
			valid_result: Result >= -32768 and Result <= 32767
		end

	poke_int16, poke_int16_native (index: INTEGER; value: INTEGER) is
			-- Write 16 bit value at offset `index', in native endian format.
		require
			valid_index: is_valid_range (index, index + 1)
			valid_value: value >= -32768 and value <= 32767
		do
			posix_poke_int16_native (ptr, index, value)
		ensure
			consistent: peek_int16_native (index) = value
		end

	poke_int16_big_endian (index: INTEGER; value: INTEGER) is
			-- Write 16 bit value at offset `index', in big endian format.
		require
			valid_index: is_valid_range (index, index + 1)
			valid_value: value >= -32768 and value <= 32767
		local
			i: INTEGER
		do
			if is_little_endian then
				i := posix_swap16 (value)
			else
				i := value
			end
			posix_poke_int16_native (ptr, index, i)
		ensure
			consistent: peek_int16_big_endian (index) = value
		end

	poke_int16_little_endian (index: INTEGER; value: INTEGER) is
			-- Write 16 bit value at offset `index', in little endian format.
		require
			valid_index: is_valid_range (index, index + 1)
			valid_value: value >= -32768 and value <= 32767
		local
			i: INTEGER
		do
			if is_big_endian then
				i := posix_swap16 (value)
			else
				i := value
			end
			posix_poke_int16_native (ptr, index, i)
		ensure
			consistent: peek_int16_little_endian (index) = value
		end


feature -- Set/get integers (32-bit data)


	peek_int32_native, peek_integer (index: INTEGER): INTEGER is
			-- Read 32 bit value at offset `index', assume its byte order
			-- is native, and return it.
		require
			valid_index: is_valid_range (index, index + 3)
		do
			Result := posix_peek_int32_native (ptr, index)
		end

	peek_int32_big_endian (index: INTEGER): INTEGER is
			-- Read 32 bit value at offset `index', assume its byte order
			-- is big endian, and return it in native format.
		require
			valid_index: is_valid_range (index, index + 3)
		local
			i: INTEGER
		do
			i := posix_peek_int32_native (ptr, index)
			if is_little_endian then
				i := posix_swap32 (i)
			end
			Result := i
		end

	peek_int32_little_endian (index: INTEGER): INTEGER is
			-- Read 32 bit value at offset `index', assume its byte order
			-- is little endian, and return it in native format.
		require
			valid_index: is_valid_range (index, index + 3)
		local
			i: INTEGER
		do
			i := posix_peek_int32_native (ptr, index)
			if is_big_endian then
				i := posix_swap32 (i)
			end
			Result := i
		end

	peek_uint32_native (index: INTEGER): INTEGER is
			-- Read 32 bit unsigned int at offset `index', assume native
			-- byte order.
		require
			valid_index: is_valid_range (index, index + 3)
		do
			Result := posix_peek_uint32_native (ptr, index)
		end

	peek_uint32_big_endian (index: INTEGER): INTEGER is
			-- Read 32 bit unsigned int at offset `index', assume its
			-- byte order is big endian, and return it in native format.
		require
			valid_index: is_valid_range (index, index + 3)
		local
			i: INTEGER
		do
			i := posix_peek_uint32_native (ptr, index)
			if is_little_endian then
				i := posix_swap32 (i)
			end
			Result := i
		end

	peek_uint32_little_endian (index: INTEGER): INTEGER is
			-- Read 32 bit unsigned int at offset `index', assume its
			-- byte order is big endian, and return it in native format.
		require
			valid_index: is_valid_range (index, index + 3)
		local
			i: INTEGER
		do
			i := posix_peek_uint32_native (ptr, index)
			if is_big_endian then
				i := posix_swap32 (i)
			end
			Result := i
		end

	poke_integer, poke_int32_native (index: INTEGER; value: INTEGER) is
			-- Write 32 bit value at offset `index', in native endian format.
		require
			valid_index: is_valid_range (index, index + 3)
		do
			posix_poke_int32_native (ptr, index, value)
		ensure
			consistent: peek_integer (index) = value
		end

	poke_int32_big_endian (index: INTEGER; value: INTEGER) is
			-- Write 32 bit value at offset `index', in big endian format.
		require
			valid_index: is_valid_range (index, index + 3)
		local
			i: INTEGER
		do
			if is_little_endian then
				i := posix_swap32 (value)
			else
				i := value
			end
			posix_poke_int32_native (ptr, index, i)
		ensure
			consistent: peek_int32_big_endian (index) = value
		end

	poke_int32_little_endian (index: INTEGER; value: INTEGER) is
			-- Write 32 bit value at offset `index', in little endian format.
		require
			valid_index: is_valid_range (index, index + 3)
		local
			i: INTEGER
		do
			if is_big_endian then
				i := posix_swap32 (value)
			else
				i := value
			end
			posix_poke_int32_native (ptr, index, i)
		ensure
			consistent: peek_int32_little_endian (index) = value
		end


feature -- Set/get characters

	peek_character (index: INTEGER): CHARACTER is
			-- return value at position `index' as a character
		require
			valid_index: is_valid_index (index)
		do
			Result := posix_peek_character (ptr, index)
		end

	poke_character (index: INTEGER; value: CHARACTER) is
			-- set character at `position' index to `value'
		require
			valid_index: is_valid_index (index)
		do
			posix_poke_character (ptr, index, value)
		ensure
			consistent: peek_character (index) = value
		end

	c_substring_with_string (dest: STRING; start_index, end_index: INTEGER) is
			-- As `c_substring' but used `dest' as the destination.
		require
			have_destination: dest /= Void
			valid_index: is_valid_range (start_index, end_index)
		local
			i: INTEGER
			c: CHARACTER
		do
			if dest.count > 0 then
				dest.wipe_out
			end
			from
				i := start_index
				c := ' '
			until
				c = '%U' or else i > end_index
			loop
				c := posix_peek_character (ptr, i)
				if c /= '%U' then
					dest.append_character (c)
					i := i + 1
				end
			end
		ensure
			dest_count_as_asked: dest.count <= end_index - start_index + 1
		end

	c_substring (start_index, end_index: INTEGER): STRING is
			-- Create a substring containing all characters from
			-- start_index up to encountering a %U or when end_index is
			-- reached, whatever happens first.
		require
			valid_index: is_valid_range (start_index, end_index)
		local
			count: INTEGER
		do
			count := end_index - start_index + 1
			Result := sh.make_with_capacity (count)
			c_substring_with_string (Result, start_index, end_index)
		ensure
			substring_not_void: Result /= Void
			substring_count_as_asked: Result.count <= end_index - start_index + 1
		end

	substring (start_index, end_index: INTEGER): STRING is
			-- Create a substring containing all characters
			-- from start_index to end_index inclusive.
		require
			valid_index: is_valid_range (start_index, end_index)
		local
			count: INTEGER
			i: INTEGER
		do
			count := end_index - start_index + 1
			Result := sh.make_with_capacity (count)
			from
				i := start_index
			until
				i > end_index
			loop
				Result.append_character (peek_character (i))
				i := i + 1
			end
		ensure
			substring_not_void: Result /= Void
			substring_count_as_asked: Result.count = end_index - start_index + 1
		end


feature -- Fill

	fill_at (start_index, a_count: INTEGER; byte: INTEGER) is
			-- Starting at position `start_index', write `byte' for `a_count' bytes
		require
			valid_range: is_valid_range (start_index, start_index + a_count-1)
			valid_byte: byte >= 0 and byte <= 255
		local
			p: POINTER
		do
			p := posix_pointer_add (ptr, start_index)
			posix_memset (p, byte, a_count)
		end


feature -- Searching

	locate_character (other: CHARACTER; start_index: INTEGER): INTEGER is
			-- Return index of `other' in buffer, or -1.
			-- Search begins at `start_index'.
		require
			valid_start_index: is_valid_index (start_index)
		local
			i: INTEGER
			Found: BOOLEAN
		do
			from
				i := start_index
			until
				Found or else i > capacity
			loop
				Found := peek_character (i) = other
				i := i + 1
			end
			if Found then
				Result := i - 1
			else
				Result := -1
			end
		ensure
			result_in_range: Result >= -1 and Result < capacity
			result_after_start_index: Result >= start_index
		end

	locate_string (other: STRING; start_index: INTEGER): INTEGER is
			-- Does buffer contain 'other'?
			-- Returns index where `other' is found.
			-- Returns -1 if not found
			-- searching starts at position `start_index'
		require
			other_not_empty: other /= Void and then not other.is_empty
			valid_index: is_valid_index (start_index)
		local
			stop: BOOLEAN
			i1, i2, i3: INTEGER
			index: INTEGER
		do
			if capacity >= other.count then
				-- the following smart search algorithm shamelessly
				-- copied from SE
				from
					index := -1
					i1 := start_index
					i3 := other.count
					i2 := i1 + i3 - 1
				invariant
					i3 = i2 - i1 + 1
				variant
					(capacity + 1) - i1
				until
					index >= 0
				loop
					if i2 >= capacity then
						index := capacity
					else
						from
							stop := False
							-- invariant
							--    i3 = i2 - i1 + 1
							-- variant
							--    (i3 + i2 + 1)
						until
							stop
						loop
							if other.item (i3) /= peek_character (i2) then
								stop := True
							else
								i3 := i3 - 1
								if i3 = 0 then
									stop := True
									index := i1
								else
									i2 := i2 - 1
								end
							end
						end
					end
					i1 := i1 + 1
					i3 := other.count
					i2 := i1 + i3 - 1
				end
				if index < capacity then
					Result := index
				else
					Result := -1
				end
			else
				Result := -1
			end
		ensure
			false_if_too_small: capacity < other.count implies Result = -1
			valid_result: Result >= -1 and Result < capacity
		end


feature -- Queries

	is_valid_index (index: INTEGER): BOOLEAN is
		do
			Result := index >= 0 and index < capacity
		end

	is_valid_range (from_index, to_index: INTEGER): BOOLEAN is
			-- Returns True if from_index..to_index is a valid and
			-- meaningfull range
		do
			Result :=
				from_index >= 0 and from_index < capacity and
				to_index >= from_index and to_index < capacity
		end


feature -- State

	size: INTEGER is
			-- alias for capacity for historical reasons.
			-- will become obsolete at some time
		obsolete "Please use capacity."
		do
			Result := capacity
		end


feature -- Low level handle functions

	do_close: BOOLEAN is
			-- Close resource, return error if any, or zero on
			-- success. This routine may never call another object, else
			-- it cannot be used safely in `dispose'.
		do
			posix_free (ptr)
			clear_handle
			Result := True
		end

	frozen unassigned_value: POINTER is
			-- The value that indicates that `handle' is unassigned.
		do
			Result := default_pointer
		end


feature {NONE} -- Counting of allocated resource

	decrease_allocated_memory (amount: INTEGER) is
			-- Decrease our idea of allocated memory. Does not call any
			-- object, so can be called from `dispose'.
		require
			amount_not_negative: amount >= 0
		external "C"
		alias "posix_decrease_allocated_memory"
		end

	decrease_resource_count is
			-- Decrease number of allocated resource by `capacity'.
			-- Due to limitations of certain Eiffel implementations, this
			-- routine may not call any other object. Calling C code is safe.
		do
			decrease_allocated_memory (capacity)
		end

	increase_resource_count is
			-- Increase number of allocated resources by `capacity'.
		do
			security.memory.increase_memory_allocation (capacity)
		end

	resource_count: INTEGER is
			-- Currently allocated number of resources. It's a global
			-- number, counting the `capacity' of all owned resources of
			-- this type.
		do
			Result := security.memory.allocated_memory
		end


feature {NONE} -- POSIX c interface

	posix_peek_character (p: POINTER; index: INTEGER): CHARACTER is
			-- read character at position `index'
		require
			valid_memory: p /= default_pointer
		external "C"
		end

	posix_peek_int16_native (p: POINTER; index: INTEGER): INTEGER is
			-- read word at position `index'
		require
			valid_memory: p /= default_pointer
		external "C"
		end

	posix_peek_int32_native (p: POINTER; index: INTEGER): INTEGER is
			-- read integer at position `index'
		require
			valid_memory: p /= default_pointer
		external "C"
		end

	posix_peek_uint8 (p: POINTER; index: INTEGER): INTEGER is
			-- read unsigned 8 bit value at position `index'
		require
			valid_memory: p /= default_pointer
		external "C"
		end

	posix_peek_uint16_native (p: POINTER; index: INTEGER): INTEGER is
			-- read unsigned 16 bit value at position `index'
		require
			valid_memory: p /= default_pointer
		external "C"
		end

	posix_peek_uint32_native (p: POINTER; index: INTEGER): INTEGER is
			-- read unsigned 32 bit value at position `index'
		require
			valid_memory: p /= default_pointer
		external "C"
		end

	posix_poke_uint8 (p: POINTER; index: INTEGER; value: INTEGER) is
			-- set byte at position `index'
		require
			valid_memory: p /= default_pointer
		external "C"
		end

	posix_poke_character (p: POINTER; index: INTEGER; value: CHARACTER) is
			-- set character at position `index'
		require
			valid_memory: p /= default_pointer
		external "C"
		end

	posix_poke_int16_native (p: POINTER; offset: INTEGER; value: INTEGER) is
			-- Put a 16 bit integer at offset `offset'.
		require
			valid_memory: p /= default_pointer
		external "C"
		end

	posix_poke_int32_native (p: POINTER; offset: INTEGER; value: INTEGER) is
			-- Put a signed 32 bit integer at offset `offset'.
		require
			valid_memory: p /= default_pointer
		external "C"
		end


feature {NONE} -- Endian operations

	posix_swap16 (i: INTEGER): INTEGER is
			-- revers byte order for lower 2 bytes, upper 2 bytes are
			-- zeroed out
		external "C"
		end

	posix_swap32 (i: INTEGER): INTEGER is
			-- revers byte order
		external "C"
		end


end
